import {
  world,
} from "@minecraft/server";

import { CHAT_COOLDOWN } from "./config.js";

export const COOLDOWNS = {};

export default class Cooldown {
    /**
     * Generates a cooldown for a player
     * @param {Player.nameTag} player player whos the cooldown is for
     */
    constructor(player) {
      this.player = player;
      this.time = Date.now() + CHAT_COOLDOWN * 1000;
  
      COOLDOWNS[this.player] = this;
      this.events = {
        tick: world.events.tick.subscribe((data) => {
          if (this.time <= Date.now()) this.expire();
        }),
        playerLeave: world.events.playerLeave.subscribe(({ playerName }) => {
          if (playerName == this.player) this.expire();
        }),
      };
    }
  
    /**
     * Caculates the time left of the cooldown
     * @returns {number}
     */
    get TIME_LEFT() {
      return Math.ceil((this.time - Date.now()) / 1000);
    }
  
    /**
     * Sends a chat reply
     * @returns {void}
     */
    reply(player) {
      return player.tell(
        `§7[§r§4Staff Plugin§7]§r§7 CoolDown §7§l>§r §7Please wait §g${this.TIME_LEFT}s §7before you can speak again§7 !`
      );
    }
  
    /**
     * Expires the cooldown
     */
    expire() {
      delete COOLDOWNS[this.player];
      for (const key in this.events) {
        world.events[key].unsubscribe(this.events[key]);
      }
    }
  }